#region usings

using System;
using System.ComponentModel.Composition;
using System.Collections.Generic;
using IrrKlang;
using IrrVec3D = IrrKlang.Vector3D;
using VVVV.PluginInterfaces.V1;
using VVVV.PluginInterfaces.V2;
using VVVV.Utils.VColor;
using VVVV.Utils.VMath;
using Vector3D = VVVV.Utils.VMath.Vector3D;
using VVVV.Core.Logging;

#endregion usings

namespace VVVV.Nodes
{ 
	#region PluginInfo
	[PluginInfo(Name = "Irrklang", Category = "sound", Version = "1", Help = "Basic template with one value in/out", Tags = "")]
	#endregion PluginInfo
	public class Irrklang : IPluginEvaluate
	{
		#region fields & pins
		[Input("File", DefaultString = "test.wav")]
		ISpread<string> FFile;
		
		[Input("Play")]
		ISpread<bool> FPlay;
		[Input("Stop")]
		ISpread<bool> FStop;
		[Input("Pause")]
		ISpread<bool> FPause;
		[Input("Play Position", DefaultValue = 1.0)]
		ISpread<float> FPlayPos;
		
		[Input("Seek Pos")]
		ISpread<bool> FSeekPos;
		
		[Input("Playback Speed", DefaultValue = 1.0)]
		ISpread<float> FPlaySpeed;
		[Input("Loop")]
		ISpread<bool> FLoop;
		
		[Input("volume", DefaultValue = 1.0)]
		ISpread<float> FVolume;
		
        [Input("Pos", DefaultValue = 0)]
		ISpread<Vector3D> FPos;
		[Input("velocity", DefaultValue = 0)]
		ISpread<Vector3D> FVelo;
		[Input("minimal distance", DefaultValue = 1.0)]
		ISpread<float> FmDist;
		
		[Input("Stop All")]
		ISpread<bool> FStopAll;
        [Input("View Pos", DefaultValue = 0)]
		ISpread<Vector3D> FViewPos;
		[Input("View Dir", DefaultValue = 0)]
		ISpread<Vector3D> FViewDir;
		[Input("Roll off factor", DefaultValue = 1.0)]
		ISpread<float> FRollOff;
		[Input("Dopler factor", DefaultValue = 1.0)]
		ISpread<float> FDoplerFactor;
		
		[Input("Dopler Distance Factor", DefaultValue = 1.0)]
		ISpread<float> FDoplerDistanceFactor;
		//[Input("Driver", EnumName = "DriverName")]
		//IDiffSpread<EnumEntry> FDriver;
        [Output("length")]
		ISpread<double> Flength;
		[Output("finished")]
		ISpread<bool> FFinish;
		[Output("Play Pos")]
		ISpread<double> FCurrentPos;
		[Output("Driver use")]
		ISpread<string> FDriver;
		ISoundEngine FEngine;	
		ISoundDeviceList Fdevice;
		Dictionary< int , ISound> FSounds;
        
		//[Import()]
		//ILogger Flogger;
		#endregion fields & pins
	 	
        public Irrklang()
        {
			//Flogger.Log(LogType.Debug, "creating sound engine");
	    	FEngine = new ISoundEngine(); 
	    	//Fdevice = new ISoundDeviceList(PlaybackDevice,SoundOutputDriver);
	    	FSounds = new Dictionary< int ,ISound >();
	    	
	    //	var DriverCount = FSounds.Fdevice.DeviceCount;
	    //	var Driver = new string[DriverCount];
	    //	Driver.add(createSoundDeviceList());
	    //	EnumManager.UpdateEnum("MyDynamicEnum", Driver[0], Driver);
			//Flogger.Log(LogType.Debug, "done"); 
			
        }
		    
		//called each frame by vvvv
		public void Evaluate(int SpreadMax)
		{  
		 
		    Flength.SliceCount = SpreadMax;
            FCurrentPos.SliceCount = SpreadMax;
            FFinish.SliceCount = SpreadMax;
            
			for (int i = 0; i < SpreadMax; i++){
			 FDriver[0] = FEngine.Name;
			 FEngine.SetListenerPosition((new IrrVec3D((float)FViewPos[0].x,(float)FViewPos[0].y,(float)FViewPos[0].z)),(new IrrVec3D((float)FViewDir[0].x,(float)FViewDir[0].y,(float)FViewDir[0].z)));
			 FEngine.SetRolloffFactor(FRollOff[0]);
			 FEngine.SetDopplerEffectParameters(FDoplerFactor[0],FDoplerDistanceFactor[0]);
			if (FPlay[i] & (!FSounds.ContainsKey(i)))
			    {
			 	    FSounds.Add(i,(FEngine.Play3D(FFile[i],(float)FPos[i].x,(float)FPos[i].y,(float)FPos[i].z,FLoop[i])));	
				   
			    }
			    
			    if (FSounds.ContainsKey(i))
		     	{   
				    FSounds[i].Position = new IrrVec3D((float)FPos[i].x,(float)FPos[i].y,(float)FPos[i].z);
				    FSounds[i].MinDistance = FmDist[i];
				    FSounds[i].PlaybackSpeed = FPlaySpeed[i];
				    FSounds[i].Volume = FVolume[i];
				    FSounds[i].Paused = FPause[i];
				    FSounds[i].Velocity = new IrrVec3D((float)FVelo[i].x,(float)FVelo[i].y,(float)FVelo[i].z);
				    Flength[i] = (FSounds[i].PlayLength)/1000.0;
				    FCurrentPos[i] = (FSounds[i].PlayPosition)/1000.0;
				    FFinish[i] = FSounds[i].Finished;
				    if(FSounds[i].Finished)
				    {
				        
				        FSounds.Remove(i);
				        FCurrentPos[i] = 0;
				    }
			    }	
			   
			    if (FSeekPos[i] & FSounds.ContainsKey(i))
			    {
                    FSounds[i].PlayPosition = (uint)(((UInt32)FPlayPos[i])*1000.0);
                }  
           			    if (FStop[i] & FSounds.ContainsKey(i))
			    {
                    FSounds[i].Stop();
                }  
			    if (FStopAll[i])
				    FEngine.StopAllSounds();
						
		    }
		}
	}
}
